//= require <dialogs/default.js>
//= require <util.js>

Ext.namespace('dSA', 'dSA.actions', 'dSA.util', 'dSA.dialogs',
              'dSA.actions.transmission');

dSA.actions.transmission.Window = Ext.extend(Ext.Window,
{
    constructor: function(config)
    {
        if (config.width === undefined)
        {
            config.width = 480;
        }

        this.maskElement = config.maskElement;

        this.dsuid = config.dsuid;

        this.startButton =
        {
            xtype: 'button',
            text: dSA._("Start"),
            scope: this,
            handler: this.startTest.createDelegate(this),
            disabled: config.deviceCount === 0
        };

        this.closeButton =
        {
            xtype: 'button',
            text: dSA._("Close"),
            scope: this,
            handler: function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
                this.close();
            }
        };


        this.viewReportButton = new Ext.Button(
        {
            width: 105,
            text: dSA._("View report"),
            handler: this.viewReport.createDelegate(this),
            disabled: true
        });

        this.downloadReportButton = new Ext.LinkButton(
        {
            width: 105,
            text: dSA._("Download report"),
            ref: 'downloadReportButton',
            href: '/dsa/html/report/taskReport?download=1',
            disabled: true
        });

        var infopanel = dSA.dialogs.CreateDefaultDeviceInfoPanel(config.name,
                                                               config.displayid,
                                                                 false);

        this.progress = new Ext.form.Label({ text: '0%' });
        this.lostMessages = new Ext.form.Label({ text: '0' });

        var rec = Ext.data.Record.create(
        [
            { name: 'id'      },
            { name: 'communicationType' },
            { name: 'average' },
            { name: 'min'     },
            { name: 'max'     }
        ]);

        var resultStore = new Ext.data.Store(
        {
            reader: new Ext.data.ArrayReader({ idIndex: 0 }, rec)
        });

        var data =
        [
            [0, dSA._("Upstream communication") +   ':', '0%', '0%', '0%'],
            [1, dSA._("Downstream communication") + ':', '0%', '0%', '0%']
        ];

        resultStore.loadData(data);

        this.resultView = new Ext.list.ListView(
        {
            reserveScrollOffset: false,
            columnResize: false,
            columnSort: false,
            height: 70,
            store: resultStore,
            columns:
            [
                {
                    header: '&#160;',
                    dataIndex: 'communicationType',
                    align: 'left',
                    id: 'commtype'
                },
                {
                    header: dSA._("Average"),
                    dataIndex: 'average',
                    width: .15,
                    align: 'right'
                },
                {
                    header: dSA._("Min"),
                    width: .11,
                    dataIndex: 'min',
                    align: 'right'
                },
                {
                    header: dSA._("Max"),
                    width: .11,
                    dataIndex: 'max',
                    align: 'right'
                }
            ]
        });

        this.checksPerDevice = new Ext.form.NumberField(
        {
             xtype: 'numberfield',
             ref: 'checksPerDevice',
             name: 'checksPerDevice',
             width: 40,
             value: 1,
             minValue: 1,
             allowDecimals: false,
             allowBlank: false,
             allowNegative: false
        });

        var formpanel = new Ext.form.FormPanel(
        {
            padding: 5,
            layout: 'table',
            layoutConfig:
            {
                columns: 2,
                tableAttrs:
                {
                    width: '100%'
                }
            },
            items:
            [
                {
                    xtype: 'label',
                    html: dSA._("Number of active devices") + ':' + '&nbsp;'
                },
                {
                    xtype: 'label',
                    text: '' + config.deviceCount
                },
                {
                    xtype: 'box',
                    colspan: 2,
                    height: 5
                },
                {
                    xtype: 'label',
                    html: dSA._("Number of test messages per device") +
                                ':' + '&nbsp;'
                },
                this.checksPerDevice,
                {
                    xtype: 'box',
                    colspan: 2,
                    height: 10
                },
                {
                    xtype: 'label',
                    html: dSA._("Progress") +  ':' + '&nbsp;'
                },
                this.progress,
                {
                    xtype: 'box',
                    colspan: 2,
                    height: 10
                },
                {
                    xtype: 'panel',
                    colspan: 2,
                    items: this.resultView
                },
                {
                    xtype: 'box',
                    colspan: 2,
                    height: 5
                },
                {
                    xtype: 'label',
                    html: dSA._("Lost messages") +  ':' + '&nbsp;'
                },
                this.lostMessages
            ]
        });

        var mainpanel =
        {
            xtype: 'panel',
            padding: 5,
            border: false,
            bodyBorder: false,
            frame: false,
            bodyCfg:
            {
                cls: 'x-window-body'
            },
            items:
            [
                infopanel,
                formpanel
            ]
        };

        Ext.apply(this,
        {
            title: dSA._("Test Transmission Quality"),
            bodyBorder: false,
            padding: 5,
            resizable: false,
            items: [ mainpanel ],
            buttonAlign: 'left',
            iconCls: 'dSA_TransmissionQualityTitle',
            buttons:
            [
                this.viewReportButton,
                this.downloadReportButton,
                '->',
                this.startButton,
                this.closeButton
            ]
        });

        dSA.actions.transmission.Window.superclass.constructor.apply(this,
                                                                     arguments);

        this.on('afterrender', 
            function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.mask();
                }
            },
            this);

        this.on('beforedestroy',
            function()
            {
                if (this.maskElement !== undefined)
                {
                    this.maskElement.unmask();
                }
                this.closed = true;
            },
            this);
    },
    updateViewFromTask: function(result)
    {
        this.lostMessages.setText(result.errors);
        var data = [];
        if ((result.avgUpstream !== undefined) &&
            (result.minUpstream !== undefined) &&
            (result.maxUpstream !== undefined))
        {
            data.push(
            [
                0, dSA._("Upstream communication") + ':',
                Math.floor((result.avgUpstream *
                 (100/dSA.config.TransmissionQuality.UPSTREAM_MAX))) + '%',
                Math.floor((result.minUpstream *
                 (100/dSA.config.TransmissionQuality.UPSTREAM_MAX))) + '%',
                Math.floor((result.maxUpstream *
                 (100/dSA.config.TransmissionQuality.UPSTREAM_MAX))) + '%'
            ]);
        }

        if ((result.avgDownstream !== undefined) &&
            (result.minDownstream !== undefined) &&
            (result.maxDownstream !== undefined))
        {
            data.push(
            [
                1, dSA._("Downstream communication") + ':',
                (100 - Math.floor((result.avgDownstream *
                    (100/dSA.config.TransmissionQuality.DOWNSTREAM_MAX))))+'%',
                (100 - Math.floor((result.minDownstream *
                    (100/dSA.config.TransmissionQuality.DOWNSTREAM_MAX))))+'%',
                (100 - Math.floor((result.maxDownstream *
                    (100/dSA.config.TransmissionQuality.DOWNSTREAM_MAX))))+'%'
            ]);
        }

        if (data.length > 0)
        {
            this.resultView.getStore().loadData(data);
        }

    },
    pollResults: function(success, task)
    {
        if (this.closed === true)
        {
            return;
        }

        if (this.isVisible() !== true)
        {
            var dtask = new Ext.util.DelayedTask(
                           dSA.functions.getTaskResultSummary, this,
                           [ task.id, this.pollResults.createDelegate(this) ]);
            dtask.delay(2500);
            return;
        }

        if (success !== true)
        {
            return;
        }

        if (task.result !== undefined)
        {
            this.updateViewFromTask(task.result);
        }

        var resultObject = this;
        dSA.functions.GetTaskStatus(task.id,
            function(success, data)
            {
                var msg;
                if (success === false)
                {
                    return;
                }
                if (data.id != 0)
                {
                    msg = dSA._("Transmission quality test is currently running.");
                    if (data.progress !== undefined)
                    {
                        resultObject.progress.setText(data.progress + '%');
                        msg = msg + ' ' + dSA._("Current progress") + ': ' +
                            data.progress + '%';
                    }
                    dSA.MessageBus.fireEvent('statusmessage', msg);
                }
		    }
        );
        
        var msg;
        if ((task['status'] === 'running') ||
            (task['status'] === 'queued') ||
            (task['status'] === 'sleeping'))
        {
            var dtask = new Ext.util.DelayedTask(
                           dSA.functions.getTaskResultSummary, this,
                           [ task.id, this.pollResults.createDelegate(this) ]);
            dtask.delay(2500);
        }
        else
        {
            if (task['status'] === 'finished')
            {
                msg = dSA._("Finished transmission quality test");
                this.progress.setText('100%');
            }
            else if (task['status'] === 'cancelled')
            {
                msg = dSA._("Transmission quality test was aborted");
            }

            if ((task['status'] === 'finished') ||
                (task['status'] === 'cancelled'))
            {
                dSA.MessageBus.fireEvent('statusmessage', msg);
                this.viewReportButton.setDisabled(false);
                this.downloadReportButton.setParams({ id: this.currentTaskID});
                this.downloadReportButton.setDisabled(false);
                this.setIconClass('dSA_TransmissionQualityTitle');
            }
        }
    },
    startTestCallback: function(success, taskID)
    {
        if (this.closed === true)
        {
            return;
        }

        if (success === true)
        {
            this.currentTaskID = taskID;
            var task = new Ext.util.DelayedTask(
                            dSA.functions.getTaskResultSummary, this,
                            [ taskID, this.pollResults.createDelegate(this) ]);
            task.delay(2500);

        }
        else
        {
            this.currentTaskID = undefined;
        }
    },
    startTest: function()
    {
        var data =
        [
            [0, dSA._("Upstream communication") +   ':', '0%', '0%', '0%'],
            [1, dSA._("Downstream communication") + ':', '0%', '0%', '0%']
        ];

        this.resultView.getStore().loadData(data);
        this.progress.setText('0%');
        this.lostMessages.setText('0');
        this.viewReportButton.setDisabled(true);
        this.downloadReportButton.setDisabled(true);
        this.setIconClass('loading-indicator-small');
        dSA.MessageBus.fireEvent('statusmessage',
                            dSA._("Starting transmission quality test..."));

        dSA.functions.CheckTransmissionQuality(this.dsuid,
                this.checksPerDevice.getValue(),
                this.startTestCallback.createDelegate(this));
    },
    viewReport: function()
    {
        var report = new Ext.Window(
        {
            title: dSA._("Transmission quality report"),
            width:  600,
            height: 400,
            layout: 'fit',
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    autoScroll: true,
                    autoLoad: '/dsa/html/report/taskReport?download=0' + '&' +
                              'id=' + this.currentTaskID
                }
            ],
            buttons:
            [
                {
                    xtype: 'button',
                    text: dSA._("Close"),
                    handler: function() { report.close(); }
                }
            ]
        });
        report.show();
    }
});

dSA.actions.transmission.Test = function(displayid, dsid, dsuid, name, maskEl)
{
    dSA.util.getNumDevices(dsuid, function(count) {
        var win = new dSA.actions.transmission.Window(
        {
            name: name,
            dsuid: dsuid,
            dsid: dsid,
            displayid: displayid,
            maskElement: maskEl,
            deviceCount: count
        });
        win.show();
    });
};
